package	jp.co.sds_corp.report;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamSource;

import jp.co.sds_corp.report.source.SourceFactory;
import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.DOMWriter;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.script.ScriptParserFactory;
import jp.co.sds_corp.util.log.CmnLogger;

import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;
import org.apache.poi.hssf.usermodel.HSSFPrintSetup;
import org.apache.poi.hssf.usermodel.HSSFSheet;

/**
<a href="http://jakarta.apache.org/poi/">Jakarta POI</a>,
<a href="http://xml.apache.org/fop/">Apache FOP</a>
を使用して、Excel帳票をPDFに出力します。
<pre>
&lt;Report
	name="ReportTest"
	type="PDF"
	template="${Report.dir}/table.xls"
	sheet="Sheet1"
	fofile="result.fo"
	debug="true"&gt;
	&lt;Source ... /&gt;
	&lt;Script ... /&gt;
&lt;/Report&gt;
</pre>
Source要素については、{@link SourceFactory}を参照してください。
Script要素については、{@link ScriptParserFactory}を参照してください。
*/
public class ExcelHTMLReport extends ExcelPDFReport
{
	static CmnLogger logger = CmnLogger.getInstance();

	/**
	* 帳票を出力します。
	* @param	out		出力先
	*/
	public void print( OutputStream out ) throws Exception {
		if ( !bPrint ) {
			print();
		}
		HSSFPrintSetup prset = result.getSheetAt( 0 ).getPrintSetup();
		try {
			DOMConfiguration root = DOMConfiguration.createDocument( "fo:root" );
			IConfiguration master = initPrint( root );
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			for ( int i = 0; i < result.getNumberOfSheets(); i++ ) {
				HSSFSheet sheet = result.getSheetAt( i );
				if ( isDebug()) {
					logger.debug( this, "print " + result.getSheetName( i ));
				}
				printSheet( root, master, result, i, sheet );
			}
			bos.close();

			StringWriter buf = new StringWriter();
			DOMWriter wr = new DOMWriter( buf );
			wr.write( root, "fo:root" );
			buf.close();

			if ( fofile != null ) {
				writeFo( buf.toString() );
			}

			FopFactory fopFactory = FopFactory.newInstance();
			if ( userconfig != null ) {
				fopFactory.setUserConfig( new File( userconfig ));
			}
			Fop fop = fopFactory.newFop( MimeConstants.MIME_PNG, out );
			TransformerFactory factory = TransformerFactory.newInstance();
			Transformer transformer = factory.newTransformer();
			Source foFile = new StreamSource( new StringReader( buf.toString() ));
			Result res = new SAXResult(fop.getDefaultHandler());
			transformer.transform(foFile, res);
			out.flush();
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}

}
